Package: PCAN
Title: Phenotype Consensus ANalysis (PCAN)
Version: 1.41.0
Date: 2016-03-24
Author: Matthew Page and Patrice Godard
Maintainer: Matthew Page <matthew.page@ucb.com> and Patrice Godard
 <patrice.godard@gmail.com>
Description: Phenotypes comparison based on a pathway consensus
        approach. Assess the relationship between candidate genes and a
        set of phenotypes based on additional genes related to the
        candidate (e.g. Pathways or network neighbors).
biocViews: Annotation, Sequencing, Genetics, FunctionalPrediction,
        VariantAnnotation, Pathways, Network
License: CC BY-NC-ND 4.0
Depends: R (>= 3.3), BiocParallel
Imports: grDevices, stats
Suggests: BiocStyle, knitr, rmarkdown, reactome.db, STRINGdb
VignetteBuilder: knitr
RoxygenNote: 5.0.1
NeedsCompilation: no
Repository: https://bioc.r-universe.dev
Date/Publication: 2026-04-28 12:42:55 UTC
RemoteUrl: https://github.com/bioc/PCAN
RemoteRef: HEAD
RemoteSha: 9293d512937f3feaab90fb9cf8e41210e481399f
Packaged: 2026-05-30 07:14:50 UTC; root
Built: R 4.6.0; ; 2026-05-30 10:11:12 UTC; unix
