# Generated by roxygen2: do not edit by hand

export(calcHpSim)
export(compareHPSets)
export(computeHpIC)
export(hpGeneHeatmap)
export(hpGeneListComp)
export(hpSetCompBestMatch)
export(hpSetCompSummary)
import(stats)
importFrom(BiocParallel,bpmapply)
importFrom(BiocParallel,bpparam)
importFrom(grDevices,colorRampPalette)
